#include "stdafx.h"

#include "GigECameraCap.h"

using namespace FlyCapture2;
using namespace cv;
using namespace std;

#ifdef _DEBUG
#pragma comment(lib, "FlyCapture2d.lib")
#pragma comment(lib, "opencv_core245d.lib")
#pragma comment(lib, "opencv_highgui245d.lib")
#else
#pragma comment(lib, "FlyCapture2.lib")
#pragma comment(lib, "opencv_core245.lib")
#pragma comment(lib, "opencv_highgui245.lib")
#endif

#ifdef _USE_TIMER
// use this one: http://www10.atwiki.jp/bambooflow/pages/249.html
#include "Timer.h"
#include "FpsCounter.h"
#endif // _USE_TIMER

int main(int argc, char* argv[])
{
	// Setup GigE Bus
	GigEBusManager bus;
	bus.Init();
	GigECameraCap *cam = new GigECameraCap[bus.num_cameras()];
	for(unsigned int i=0; i<bus.num_cameras(); i++){
		cam[i].Init(bus.guid(i));
	}

	for(unsigned int i=0; i<bus.num_cameras(); i++){
		char winname[256];
		sprintf(winname,"cam%d",i);
		namedWindow( winname );
	}

	// Setup GigE Camera
	Mat *show = new Mat[bus.num_cameras()];
	
#ifdef _USE_TIMER
	FpsCounter fps;
	Timer t;
	t.start();
#endif // _USE_TIMER
	
	for(int i=0; i<1000; i++){
#ifdef _OPENMP
#pragma omp parallel for
#endif
		for(int j=0; j<(int)bus.num_cameras(); j++){
			cam[j].grap(show[j]);
		}

		for(unsigned int j=0; j<bus.num_cameras(); j++){
			char winname[256];
			sprintf(winname,"cam%d",j);
			imshow( winname ,show[j]); //x
			//cvShowImage(itoa(j,buf,10),cam[j].iplgrap());
		}

		waitKey(1);
#ifdef _USE_TIMER
		fps.update();
		std::cout << "FrameRate = " << fps.getFramerate() << " FPS" << std::endl;
#endif // _USE_TIMER
	}

	printf( "Done! Press Enter to exit...\n" );
    getchar();

	delete [] cam;
    return 0;
}
